

In a further attempt to prevent dirty pages from being written out from the
LRU, don't write them is they were referenced.  This gives those pages
another trip around the inactive list.  So more of them are written via
balance_dirty_pages().

It speeds up an untar-of-five-kernel trees by 5% on a 256M box.


 mm/vmscan.c |    7 ++++++-
 1 files changed, 6 insertions(+), 1 deletion(-)

diff -puN mm/vmscan.c~vmscan-defer-writepage mm/vmscan.c
--- 25/mm/vmscan.c~vmscan-defer-writepage	2003-07-26 22:14:23.000000000 -0700
+++ 25-akpm/mm/vmscan.c	2003-07-26 22:14:23.000000000 -0700
@@ -238,6 +238,7 @@ static int may_write_to_queue(struct bac
 /*
  * shrink_list returns the number of reclaimed pages
  */
+int akpm;
 static int
 shrink_list(struct list_head *page_list, unsigned int gfp_mask,
 		int *max_scan, int *nr_mapped)
@@ -254,6 +255,7 @@ shrink_list(struct list_head *page_list,
 	while (!list_empty(page_list)) {
 		struct page *page;
 		int may_enter_fs;
+		int referenced;
 
 		page = list_entry(page_list->prev, struct page, lru);
 		list_del(&page->lru);
@@ -273,7 +275,8 @@ shrink_list(struct list_head *page_list,
 			goto keep_locked;
 
 		pte_chain_lock(page);
-		if (page_referenced(page) && page_mapping_inuse(page)) {
+		referenced = page_referenced(page);
+		if (referenced && page_mapping_inuse(page)) {
 			/* In active use or really unfreeable.  Activate it. */
 			pte_chain_unlock(page);
 			goto activate_locked;
@@ -333,6 +336,8 @@ shrink_list(struct list_head *page_list,
 		 * See swapfile.c:page_queue_congested().
 		 */
 		if (PageDirty(page)) {
+			if (akpm && referenced)
+				goto keep_locked;
 			if (!is_page_cache_freeable(page))
 				goto keep_locked;
 			if (!mapping)

_
